package cn.idaddy.open.demo;

import cn.idaddy.open.demo.common.Log;
import cn.idaddy.open.demo.common.MementoCaretaker;
import cn.idaddy.open.demo.common.Strings;
import cn.idaddy.open.demo.common.ui.Dialogs;
import cn.idaddy.open.demo.common.ui.ErrorBox;
import cn.idaddy.open.demo.common.ui.Progressable;
import cn.idaddy.open.demo.service.AppSign;
import cn.idaddy.open.demo.service.OpenService;
import cn.idaddy.open.demo.service.dto.AudioDto;
import cn.idaddy.open.demo.service.dto.DeviceTokenDto;

import com.fasterxml.jackson.databind.node.ObjectNode;
import javafx.application.Application;
import javafx.application.Platform;
import javafx.concurrent.Task;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Node;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.layout.*;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import javafx.scene.text.TextBoundsType;
import javafx.stage.Stage;

import java.time.Clock;
import java.util.List;

public class App extends Application {
    AudioTable audioTable = new AudioTable(getHostServices());

    Progressable<BorderPane> progressable = new Progressable<>(new BorderPane());
    ErrorBox Validator = new ErrorBox();
    TextField app_id = textField("app_id");
    TextField app_secert = password("app_secert"); 
    TextField device_id = textField("device_id");
    
    TextField device_token_value=readOnlyTextField("device_token_value");
    TextField device_token_created_at=readOnlyTextField("device_token_created_at");
    TextField device_token_expires_at=readOnlyTextField("device_token_expires_at");

    Button refresh_device_token = new Button("refresh device token");
    {
        refresh_device_token.setOnAction(e -> {
            Task<Void> task = new Task<Void>() {
                @Override
                public Void call() throws InterruptedException {
                    try {
                        if(!checkAppSecret()){
                            return null;
                        }
                        if(!checkDeviceId()){
                            return null;
                        }

                        AppSign sign = AppSign.sign(get_app_id(), get_app_secert(), Clock.systemDefaultZone());
                        DeviceTokenDto token = openService.refresh_device_token(sign, device_id.getText());
                        Platform.runLater(() -> {
                            set_device_token(token);
                        });
                    } catch (Exception e) {
                        Dialogs.showError(e);
                    }
                    return null;
                }
            };
            progressable.run(task);
        });
    }
    private OpenService openService = new OpenService();
    private MementoCaretaker caretaker = new MementoCaretaker();

    private Scene scene = new Scene(createContent(), 1200, 800);
    {
        scene.getStylesheets().add(App.class.getResource("application.css").toExternalForm());
    }

    @Override
    public void start(Stage primaryStage) {
        try {
            //全局异常捕获
            Thread.setDefaultUncaughtExceptionHandler((Thread t, Throwable e) -> {
                Platform.runLater(() -> {
                    Dialogs.showError(e);
                });
            });

            //恢复状态
            restoreMemento(caretaker.load());

            primaryStage.setTitle("工爸开放平台DEMO");
            setUserAgentStylesheet(STYLESHEET_MODENA);
            primaryStage.setScene(scene);
            primaryStage.show();
        } catch (Exception e) {
            Dialogs.showError(e);
        }
    }


    @Override
    public void stop() {
        Log.info("Stage is closing");

        // Save 状态
        caretaker.save(saveMemento());
    }

    public Parent createContent() {
        BorderPane root = progressable.getContent();
        root.getStyleClass().add("root");
        root.setTop(
                topBar(
                        hgrowPaddingPane(),
                        searchBox()
                )
        );

        root.setLeft(
                leftBox(
                        labelTitle("输入您的授权密钥"),
                        label("app_id"), app_id,
                        label("app_secert"), app_secert,
                        label("device_id"), device_id,
                        refresh_device_token,
                        hSeparator(),
                        labelTitle("设备授权信息"),
                        label("device_token"), device_token_value,
                        label("created_at"), device_token_created_at,
                        label("expires_at"), device_token_expires_at,
                        hSeparator(),
                        labelTitle("相关链接"),
                        link("工爸官网","http://open.idaddy.cn/"),
                        link("工爸开放平台","http://open.idaddy.cn/"),
                        link("工爸开放平台文档","http://open.idaddy.cn/doc-v2/index.html")

                )
        );

        root.setCenter(audioTable.getTableView());

        return progressable.getWarpedContent();
    }


    private Hyperlink link(String text,String link){
        Hyperlink result = new Hyperlink(text);
        result.setOnAction(t -> getHostServices().showDocument(link));
        return result;
    }
    private SearchBox searchBox() {
        SearchBox result =  new SearchBox("search_audio", "音频搜索：公主");
        result.setId("search_audio");
        result.getTextBox().setOnAction(e->{
            Task<Void> task = new Task<Void>() {
                @Override
                public Void call() throws InterruptedException {
                    try {
                        if(!checkAppSecret()){
                            return null;
                        }
                        if(!checkDeviceToken()){
                            return null;
                        }
                        AppSign sign = AppSign.sign(get_app_id(), get_app_secert(), Clock.systemDefaultZone());
                        List<AudioDto> audios = openService.find_audio(sign,get_device_id(), result.getTextBox().getText(), 0, 10);
                        Platform.runLater(() -> audioTable.setItems(audios,device_token_value.getText()));
                    } catch (Exception e) {
                        Platform.runLater(() -> {
                            Dialogs.showError(e);
                        });
                    }
                    return null;
                }
            };
            progressable.run(task);
        });
        return result;
    }


    private ToolBar topBar(Node... children) {
        ToolBar toolBar = new ToolBar(children);
        toolBar.setId("top_tool_bar");
        return toolBar;
    }

    private Pane hgrowPaddingPane() {
        Pane paddingPane = new Pane();
        HBox.setHgrow(paddingPane, Priority.ALWAYS);
        return paddingPane;
    }

    private boolean checkAppSecret(){
        app_id.setText(Strings.trim(app_id.getText()));
        if (get_app_id() == null | get_app_id().isEmpty()) {
            Validator.show(app_id, "请输入app_id");
            return false;
        }

        app_secert.setText(Strings.trim(app_secert.getText()));
        if (get_app_secert() == null | get_app_secert().isEmpty()) {
            Validator.show(app_secert, "请输入app_secert");
            return false;
        }

        return true;
    }
    private boolean checkDeviceToken(){
        if (Strings.isNullOrEmpty(device_token_value.getText())) {
            Validator.show(refresh_device_token, "请先获取device_token");
            return false;
        }
        return true;
    }
    private boolean checkDeviceId(){
        device_id.setText(Strings.trim(device_id.getText()));
        if (Strings.isNullOrEmpty(device_id.getText())) {
            Validator.show(device_id, "请输入device_id");
            return false;
        }
        return true;
    }


    public void set_device_token(DeviceTokenDto value) {
        device_token_value.setText(value.device_token);
        device_token_created_at.setText(value.get_created_at_string());
        device_token_expires_at.setText(value.get_expires_at_string());
    }

    public String get_app_id() {
        return app_id.getText();
    }

    public void set_app_id(String value) {
        app_id.setText(value);
    }

    public String get_app_secert() {return app_secert.getText();}

    public void set_app_secert(String value) {
        app_secert.setText(value);
    }

    public String get_device_id() {return device_id.getText();}

    public void set_device_id(String value) {
        device_id.setText(value);
    }

    private TextField readOnlyTextField(String id) {
        TextField result = new TextField();
        result.setId(id);
        result.setEditable(false);
        result.getStyleClass().add("readonly");
        return result;
    }
    private TextField textField(String id) {
        TextField result = new TextField();
        result.setId(id);
        return result;
    }
    private PasswordField password(String id) {
    	    PasswordField result = new PasswordField();
        result.setId(id);
        return result;
    }
    
    private Separator hSeparator() {
        Separator sep2 = new Separator();
        sep2.setPrefHeight(50);
        return sep2;
    }

    private VBox leftBox(Node... children) {
        VBox vBox = new VBox();
        vBox.setId("leftBox");

        vBox.setSpacing(3);
        vBox.setPadding(new Insets(10, 3, 10, 3));
        vBox.setAlignment(Pos.TOP_LEFT);
        vBox.getChildren().addAll(children);
        return vBox;
    }
    private Node labelTitle(String label) {
        Text result = new Text(label); //20, 110,
//        resul.setFill(Color.WHITESMOKE);
//        resul.setEffect(new Lighting());
        result.setBoundsType(TextBoundsType.VISUAL);
        result.setFont(Font.font(Font.getDefault().getFamily(), 24));
        return result;
    }

    private Node label(String label) {
        Text resul = new Text(label); //20, 110,
//        resul.setFill(Color.WHITESMOKE);
//        resul.setEffect(new Lighting());
        return resul;
    }

    public void restoreMemento(ObjectNode memento) {
        set_app_id(memento.path("secert").path("app_id").asText());
        set_app_secert(memento.path("secert").path("app_secert").asText());
        set_device_id(memento.path("secert").path("device_id").asText());
    }

    public ObjectNode saveMemento() {
        ObjectNode memento = caretaker.mapper().createObjectNode();
        ObjectNode secert = memento.putObject("secert");
        secert.put("app_id", get_app_id());
        secert.put("app_secert", get_app_secert());
        secert.put("device_id", get_device_id());
        return memento;
    }

    public static void main(String[] args) {
        launch(args);
    }

}
