package cn.idaddy.open.demo;

import java.util.List;

import cn.idaddy.open.demo.common.Strings;
import cn.idaddy.open.demo.service.OpenService;
import cn.idaddy.open.demo.service.dto.AudioChapterDto;
import cn.idaddy.open.demo.service.dto.AudioDto;
import javafx.application.HostServices;
import javafx.application.Platform;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;
import javafx.scene.control.TreeItem;
import javafx.scene.control.TreeTableCell;
import javafx.scene.control.TreeTableColumn;
import javafx.scene.control.TreeTableView;
import javafx.scene.control.cell.TreeItemPropertyValueFactory;
import javafx.scene.layout.HBox;

/**
 * Created by chenpeng on 03/07/2017.
 */
public class AudioTable {

    private final TreeTableView<AudioTableModel> treeTableView = new TreeTableView<>();


    public AudioTable(HostServices hostServices){
        treeTableView.setShowRoot(false);

        {
            TreeTableColumn<AudioTableModel, String> column=treeColumn("name", 200, "name");
            treeTableView.getColumns().add(column);
        }
        {
            TreeTableColumn<AudioTableModel, String> column=treeColumn("类型", 100, "type");
            treeTableView.getColumns().add(column);
        }
        {
            TreeTableColumn<AudioTableModel, String> column=treeColumn("音频链接-原始", 200, "play_url");
            column.setCellFactory(tc -> new ReadonlyTextTreeCell());
            treeTableView.getColumns().add(column);
        }
        {
            TreeTableColumn<AudioTableModel, String> column=treeColumn("音频链接-已设备授权", 300, "play_url_authorized");
            column.setCellFactory(tc -> new PlayUrlAuthorizedTreeCell(hostServices));
            treeTableView.getColumns().add(column);
        }

    }

    public Node getTableView(){
        return treeTableView;
    }


    private TreeTableColumn<AudioTableModel, String> treeColumn (String title, int minWidth, String property){
        final TreeTableColumn<AudioTableModel, String> column = new TreeTableColumn<>();
        column.setText(title);
        column.setEditable(false);
        column.setMinWidth(minWidth);
        column.setCellValueFactory(new TreeItemPropertyValueFactory<>(property));
        return column;

    }

    public void setItems(List<AudioDto> audios, String device_token) {
        final TreeItem<AudioTableModel> rootItem = new TreeItem<>();
        for (AudioDto audio : audios) {
            TreeItem<AudioTableModel> audioItem = new TreeItem<>(new AudioTableModel(audio,device_token));
            rootItem.getChildren().add(audioItem);
            if(audio.hasChapter()&&audio.chapters!=null){
                for (AudioChapterDto chapter:audio.chapters) {
                    TreeItem<AudioTableModel> chapterItem = new TreeItem<>(new AudioTableModel(chapter,device_token));
                    audioItem.getChildren().add(chapterItem);
                }
            }
        }
        rootItem.setExpanded(true);

        this.treeTableView.setRoot(rootItem);
    }

    public static class AudioTableModel {
        private StringProperty type =new SimpleStringProperty();
        private StringProperty name=new SimpleStringProperty();
        private StringProperty play_url=new SimpleStringProperty();
        private StringProperty play_url_authorized=new SimpleStringProperty();

        public AudioTableModel(AudioDto audio, String device_token) {
            this.nameProperty().set(audio.name);
            this.play_urlProperty().set(audio.play_url);
            this.typeProperty().set(audio.hasChapter()?"多章节音频":"独立音频");
            this.play_url_authorizedProperty().set(OpenService.addDeviceToken(audio.play_url,device_token));
        }

        public AudioTableModel(AudioChapterDto chapter, String device_token) {
            this.nameProperty().set(chapter.name);
            this.play_urlProperty().set(chapter.play_url);
            this.typeProperty().set("章节");
            this.play_url_authorizedProperty().set(OpenService.addDeviceToken(chapter.play_url,device_token));

        }


        public StringProperty typeProperty() {
            return type;
        }
        public StringProperty nameProperty() {
            return name;
        }
        public StringProperty play_urlProperty() {return play_url;}
        public StringProperty play_url_authorizedProperty() {return play_url_authorized;}

     }
    private static class ReadonlyTextTreeCell extends TreeTableCell<AudioTableModel,String> {
        private final TextField textField;
        public ReadonlyTextTreeCell() {
            this.textField=readonlyTextField();
        }
        private TextField readonlyTextField() {
            TextField result = new TextField();
//            result.setId(id);
            result.setEditable(false);
            result.getStyleClass().add("readonly");
            return result;
        }

        @Override
        protected void updateItem(String item, boolean empty) {
            super.updateItem(item, empty);
            textField.setText(item);
            setGraphic(empty ? null : textField);
        }

    }

    private static class PlayUrlAuthorizedTreeCell extends TreeTableCell<AudioTableModel,String> {

        private final HostServices hostServices;


        public PlayUrlAuthorizedTreeCell(HostServices hostServices) {
            this.hostServices=hostServices;
        }
        private TextField readonlyTextField(String text) {
            TextField result = new TextField();
//            result.setId(id);
            result.setText(text);
            result.setEditable(false);
            result.getStyleClass().add("readonly");
            return result;
        }

        @Override
        protected void updateItem(String item, boolean empty) {
            super.updateItem(item, empty);
            setGraphic(empty ? null : box(
                    readonlyTextField(item), button_play(item)
                    )
            );
        }
        private HBox box(Node... children) {
            HBox vBox = new HBox();
            vBox.getChildren().addAll(children);
            return vBox;
        }
        private Button button_play(String uri) {
            Button result = new Button("打开音频");
            result.setPrefWidth(100);
            if(Strings.isNullOrEmpty(uri)){
                result.setDisable(true);
            }else{
                result.setOnAction(e -> {
                    Platform.runLater(() -> {
                        hostServices.showDocument(uri);
                    });

                });
            }
            return result;
        }

    }

}